require 'statsd-ruby'

module StatsdTaggable
  class Client
    extend Forwardable
    def_delegators :@client, :gauge, :increment, :decrement, :count, :set, :timing, :time

    def initialize(config)
      @host = config[:host] || 'localhost'
      @port = (config[:port] || 8125).to_i
      @app_name = config[:app_name] || "my_app"
      @tag_prefix = config[:tag_prefix] || '_t_'
      @hostname = `hostname`.split(".").first
      @client = Statsd.new(@host, @port)
      if config.include? :logger
        Statsd.logger = config[:logger]
      end
      @default_tags = {
          'host' => @hostname,
          'app' => @app_name
      }
    end


    def gauge_with_tags (metric, value, tags, sample_rate = 1)
      with_tagged_name(metric, tags) { |new_metric_name| @client.gauge(new_metric_name, *[value, sample_rate]) }
    end

    def increment_with_tags (metric, tags, sample_rate = 1)
      with_tagged_name(metric, tags) { |new_metric_name| @client.increment(new_metric_name, sample_rate) }
    end

    def decrement_with_tags (metric, tags, sample_rate = 1)
      with_tagged_name(metric, tags) { |new_metric_name| @client.decrement(new_metric_name, sample_rate) }
    end

    def count_with_tags (metric, count, tags, sample_rate = 1)
      with_tagged_name(metric, tags) { |new_metric_name| @client.count(new_metric_name, count, sample_rate) }
    end

    def set_with_tags (metric, value, tags, sample_rate = 1)
      with_tagged_name(metric, tags) { |new_metric_name| @client.set(new_metric_name, value, sample_rate) }
    end

    def timing_with_tags (metric, tim, tags, sample_rate = 1)
      with_tagged_name(metric, tags) { |new_metric_name| @client.timing(new_metric_name, tim, sample_rate) }
    end

    def time_with_tags (metric, tags, sample_rate = 1, &block)
      with_tagged_name(metric, tags) { |new_metric_name| @client.time(new_metric_name, sample_rate, &block) }
    end

    protected

    def with_tagged_name (metric, tags)
      yield encode_tags(metric, tags.merge(@default_tags))
    end

    def encode_tags(metric, tags)
      tags.inject([metric]) do |memo, (k, v)|
        memo << "#{@tag_prefix}#{sanitize(k)}.#{sanitize(v)}"
      end.join(".")
    end

    def sanitize v
      v.to_s.gsub ".", "_"
    end
  end

  class BasicMetrics
    def initialize(c)
      @client = c
    end

    def emit(tags = {})

      # TODO: ensure shelling out doesn't happen too often, e.g.,
      # preserve a timestamp of the last shell-out, and refuse to shell-out
      # again unless the timestamp is old enough (e.g. 10 seconds)

      rss, pcpu = `ps -o rss=,pcpu= -p #{Process.pid}`.split
      rss = rss.to_i
      pcpu = pcpu.to_f

      @client.gauge_with_tags "statsd.apps.rss", rss, tags
      @client.gauge_with_tags "statsd.apps.pcpu", pcpu, tags
    end
  end
end
